@@ -379,20 +379,17 @@ std::cerr << "article " << a.message_id << " references " << s << std::endl;
 void
 DataImpl :: load_part (const Quark          & group,
                        const Quark          & mid,
-                       size_t                 number,
-                       size_t                 lines,
-                       Article::Part        & new_part)
+                       int                    number,
+                       const StringView     & part_mid,
+                       unsigned long          lines,
+                       unsigned long          bytes)
 {
    GroupHeaders * h = get_group_headers (group);
    Article * a (h->find_article (mid));
+   pan_return_if_fail (a != 0);
 
-   pan_return_if_fail (a != NULL);
-
-   Article::Part& old_part (a->get_part (number));
-   if (old_part.empty()) {
-       old_part.swap (new_part);
-       a->lines += lines;
-   }
+   if (a->add_part (number, part_mid, bytes))
+     a->lines += lines;
 }
 
 namespace
@@ -472,6 +469,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
       in->getline (line);
       //const unsigned long article_qty = view_to_ul (line); /* unused */
       const time_t now (time (0));
+      PartBatch part_batch;
       for (;;)
       {
         // look for the beginning of an Article record.
@@ -513,7 +511,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
           if (tok.pop_token(server_tok,':') && tok.pop_token(group_tok,':')) {
             target_it->server = server_tok;
             target_it->group = group_tok.len==1 ? xref_lookup[(int)*group_tok.str] : Quark(group_tok);
-            target_it->number = view_to_ul (tok);
+            target_it->number = atoi (tok.str);
             const Server * server (find_server (target_it->server));
             if (server && ((!server->article_expiration_age) || (days_old <= server->article_expiration_age)))
               ++target_it;
@@ -534,11 +532,10 @@ DataImpl :: load_headers (const DataIO   & data_io,
           s.ltrim(); s.pop_token (tok); total_part_count = atoi(tok.str);
           s.ltrim(); s.pop_token (tok); found_part_count = atoi(tok.str);
         }
-        s.ltrim(); if (s.pop_token (tok)) a.lines = view_to_ul (tok); // this field was added in 0.115
-        if (!expired)
-          a.set_part_count (total_part_count);
+        s.ltrim(); if (s.pop_token (tok)) a.lines = atoi (tok.str); // this field was added in 0.115
 
         // found parts...
+        part_batch.init (a.message_id, total_part_count, found_part_count);
         for (int i(0), count(found_part_count); i<count; ++i)
         {
           const bool gotline (in->getline (s));
@@ -548,19 +545,25 @@ DataImpl :: load_headers (const DataIO   & data_io,
             StringView tok;
             s.ltrim ();
             s.pop_token (tok);
-            const unsigned long number (view_to_ul (tok));
-            if (!a.has_part(number)) { // corrupted entry
+            const int number (atoi (tok.str));
+            if (number > total_part_count) { // corrupted entry
               expired = true;
               break;
             }
-            Article::Part& p (a.get_part (number));
+            StringView part_mid;
+            unsigned long part_bytes (0);
             s.ltrim ();
-            s.pop_token (tok);
-            p.set_message_id (a.message_id, (tok.len==1 && *tok.str=='"') ? a.message_id.to_view() : tok);
-            s.pop_token(tok); p.bytes = view_to_ul (tok);
-            if (s.pop_token(tok)) a.lines += view_to_ul (tok); // this field was removed in 0.115
+            s.pop_token (part_mid);
+            if (part_mid.len==1 && *part_mid.str=='"')
+              part_mid = a.message_id.to_view ();
+            s.pop_token(tok); part_bytes = view_to_ul (tok);
+            part_batch.add_part (number, part_mid, part_bytes);
+
+            if (s.pop_token(tok)) a.lines += atoi (tok.str); // this field was removed in 0.115
           }
         }
+        if (!expired)
+          a.set_parts (part_batch);
 
         // add the article to the group if it hasn't all expired
         if (expired)
@@ -685,37 +688,36 @@ DataImpl :: save_headers (DataIO                       & data_io,
   }
   else
   {
-    std::ostream * out (data_io.write_group_headers(group));
+    std::ostream& out (*data_io.write_group_headers(group));
 
-    *out << "#\n"
-            "# This file has three sections.\n"
-            "#\n"
-            "# A. A shorthand table for the most frequent groups in the xrefs.\n"
-            "#    The first line tells the number of elements to follow,\n"
-            "#    then one line per entry with a one-character shorthand and full name.\n"
-            "#\n"
-            "# B. A shorthand table for the most freqent author names.\n"
-            "#    This is formatted just like the other shorthand table.\n"
-            "#    (sorted by post count, so it's also a most-frequent-posters list...)\n"
-            "#\n"
-            "# C. The group's headers section.\n"
-            "#    The first line tells the number of articles to follow,\n"
-            "#    then articles which each have the following lines:\n"
-            "#    1. message-id\n"
-            "#    2. subject\n"
-            "#    3. author\n"
-            "#    4. references. This line is omitted if the Article has an empty References header.\n"
-            "#    5. time-posted. This is a time_t (see http://en.wikipedia.org/wiki/Unix_time)\n"
-            "#    6. xref line, server1:group1:number1 server2:group2:number2 ...\n"
-            "#    7. has-attachments [parts-total-count parts-found-count] line-count\n"
-            "#       If has-attachments isn't 't' (for true), fields 2 and 3 are omitted.\n"
-            "#       If fields 2 and 3 are equal, the article is `complete'.\n"
-            "#    8. One line per parts-found-count: part-index message-id byte-count\n"
-            "#       a 'message-id' of '\"' is shorthand meaning use the message-id from line 1.\n"
-            "#\n";
+    out << "#\n"
+           "# This file has three sections.\n"
+           "#\n"
+           "# A. A shorthand table for the most frequent groups in the xrefs.\n"
+           "#    The first line tells the number of elements to follow,\n"
+           "#    then one line per entry with a one-character shorthand and full name.\n"
+           "#\n"
+           "# B. A shorthand table for the most freqent author names.\n"
+           "#    This is formatted just like the other shorthand table.\n"
+           "#    (sorted by post count, so it's also a most-frequent-posters list...)\n"
+           "#\n"
+           "# C. The group's headers section.\n"
+           "#    The first line tells the number of articles to follow,\n"
+           "#    then articles which each have the following lines:\n"
+           "#    1. message-id\n"
+           "#    2. subject\n"
+           "#    3. author\n"
+           "#    4. references. This line is omitted if the Article has an empty References header.\n"
+           "#    5. time-posted. This is a time_t (see http://en.wikipedia.org/wiki/Unix_time)\n"
+           "#    6. xref line, server1:group1:number1 server2:group2:number2 ...\n"
+           "#    7. has-attachments [parts-total-count parts-found-count] line-count\n"
+           "#       If has-attachments isn't 't' (for true), fields 2 and 3 are omitted.\n"
+           "#       If fields 2 and 3 are equal, the article is `complete'.\n"
+           "#    8. One line per parts-found-count: part-index message-id byte-count\n"
+           "#\n";
 
     // lines moved from line 8 to line 7 in 0.115, causing version 2
-    *out << "2\t # file format version number\n";
+    out << "2\t # file format version number\n";
 
     // xref lookup section
     frequency_t frequency;
@@ -724,7 +726,7 @@ DataImpl :: save_headers (DataIO                       & data_io,
         ++frequency[xit->group];
     QuarkToSymbol xref_qts;
     build_qts (frequency, xref_qts);
-    xref_qts.write (*out, "xref shorthand count");
+    xref_qts.write (out, "xref shorthand count");
 
     // author lookup section
     frequency.clear ();
@@ -732,12 +734,11 @@ DataImpl :: save_headers (DataIO                       & data_io,
       ++frequency[(*ait)->author];
     QuarkToSymbol author_qts;
     build_qts (frequency, author_qts);
-    author_qts.write (*out, "author shorthand count");
+    author_qts.write (out, "author shorthand count");
 
     // header section
-    *out << articles.size() << endl;
+    out << articles.size() << endl;
     std::string references;
-    std::string part_mid;
     foreach_const (std::vector<Article*>, articles, ait)
     {
       ++article_count;
@@ -747,54 +748,53 @@ DataImpl :: save_headers (DataIO                       & data_io,
       h->build_references_header (a, references);
 
       // message-id, subject, author
-      *out << message_id << "\n\t"
-           << a->subject << "\n\t"
-           << author_qts(a->author) << "\n\t";
+      out << message_id << "\n\t"
+          << a->subject << "\n\t"
+          << author_qts(a->author) << "\n\t";
       // references line *IF* the article has a References header
       if (!references.empty())
-        *out << references << "\n\t";
+        out << references << "\n\t";
 
       // date
-      *out << a->time_posted << "\n\t";
+      out << a->time_posted << "\n\t";
 
       // xref
-      foreach_const (Xref, a->xref, xit)
-        *out << xref_qts(xit->server) << ':' << xref_qts(xit->group) << ':' << xit->number << ' ';
-      *out << "\n\t";
+      foreach_const (Xref, a->xref, xit) {
+        out << xref_qts(xit->server);
+        out.put(':');
+        out << xref_qts(xit->group);
+        out.put(':');
+        out << xit->number;
+        out.put(' ');
+      }
+      out << "\n\t";
 
       // is_binary [total_part_count found_part_count]
-      *out << (a->is_binary ? 't' : 'f');
+      out.put (a->is_binary ? 't' : 'f');
       if (a->is_binary) {
-        int foundPartCount (0);
-        foreach_const (Article::parts_t, a->parts, pit)
-          if (!pit->empty())
-            ++foundPartCount;
-        *out << ' ' << a->get_part_count() << ' ' << foundPartCount;
+        out.put(' ');
+        out << a->get_total_part_count();
+        out.put(' ');
+        out << a->get_found_part_count();
       }
-      *out << ' ' << a->lines << '\n';
+      out.put(' ');
+      out << a->lines;
+      out.put('\n');
 
       // one line per foundPartCount (part-index message-id bytes lines)
-      int number (0);
-      bool first (true);
-      foreach_const (Article::parts_t, a->parts, pit) {
-        ++number;
-        if (!pit->empty()) {
-          ++part_count;
-          const Article::Part& p (*pit);
-          *out << '\t' << number << ' ';
-          p.get_message_id (message_id, part_mid);
-          if (message_id.to_view() == part_mid)
-            *out << '"';
-          else
-            *out << part_mid;
-          *out << ' ' << p.bytes << '\n';
-           first = false;
-        }
+      for (Article::part_iterator pit(a->pbegin()), end(a->pend()); pit!=end; ++pit) {
+        out.put('\t'); out << pit.number();
+        out.put (' ');
+        out << pit.mid();
+        out.put (' ');
+        out << pit.bytes();
+        out.put ('\n');
+        ++part_count;
       }
     }
 
-    success = !out->fail ();
-    data_io.write_done (out);
+    success = !out.fail ();
+    data_io.write_done (&out);
     save_group_xovers (data_io);
   }
 
@@ -815,8 +815,8 @@ DataImpl :: save_headers (DataIO& data_io, const Quark& group) const
    const GroupHeaders * h (get_group_headers (group));
    std::vector<Article*> articles;
    foreach_const (nodes_t, h->_nodes, it)
-      if (it->second->_article)
-         articles.push_back (it->second->_article);
+     if (it->second->_article)
+       articles.push_back (it->second->_article);
 
    unsigned long part_count (0ul);
    unsigned long article_count (0ul);
