@@ -112,6 +112,7 @@ const char * fallback_ui_file =
 "      </menu>\n"
 "      <menu action='article-actions-menu'>\n"
 "        <menuitem action='save-articles' />\n"
+"        <menuitem action='save-articles-from-nzb' />\n"
 "        <separator />\n"
 "        <menuitem action='read-selected-article' />\n"
 "        <menuitem action='download-selected-article' />\n"
@@ -136,7 +137,6 @@ const char * fallback_ui_file =
 "        <menuitem action='supersede-article' />\n"
 "        <menuitem action='cancel-article' />\n"
 "      </menu>\n"
-//"    </menu>\n"
 "    <menu action='help-menu'>\n"
 "      <menuitem action='pan-web-page' />\n"
 "      <menuitem action='tip-jar' />\n"
@@ -173,6 +173,7 @@ const char * fallback_ui_file =
 "\n"
 "  <popup name='header-pane-popup'>\n"
 "    <menuitem action='save-articles' />\n"
+"    <menuitem action='save-articles-from-nzb' />\n"
 "    <separator />\n"
 "    <menuitem action='read-selected-article' />\n"
 "    <menuitem action='download-selected-article' />\n"
