@@ -349,8 +349,6 @@ gnksa_check_domain_literal (const StringView& domain)
 int
 GNKSA :: check_domain (const StringView& domain)
 {
-   debug (domain);
-
    if (domain.empty())
       return GNKSA::ZERO_LENGTH_LABEL;
 
@@ -470,6 +468,8 @@ namespace
 
 namespace
 {
+   enum { ADDRTYPE_ROUTE, ADDRTYPE_OLDSTYLE };
+
    int split_from (const StringView   & from,
                    StringView         & addr,
                    StringView         & name,
@@ -486,13 +486,13 @@ namespace
 
       // empty string
       if (myfrom.empty()) {
-         addrtype = GNKSA::ADDRTYPE_OLDSTYLE;
+         addrtype = ADDRTYPE_OLDSTYLE;
          return GNKSA::LPAREN_MISSING;
       }
 
       if (myfrom.back() == '>') // Charles Kerr <charles@rebelbase.com>
       {
-         addrtype = GNKSA::ADDRTYPE_ROUTE;
+         addrtype = ADDRTYPE_ROUTE;
 
          // get address part
          char * begin = myfrom.strrchr ('<');
@@ -519,7 +519,7 @@ namespace
       }
       else if ((lparen = myfrom.strchr('(')) != NULL) // charles@rebelbase.com (Charles Kerr)
       {
-         addrtype = GNKSA::ADDRTYPE_OLDSTYLE;
+         addrtype = ADDRTYPE_OLDSTYLE;
 
          // address part
          StringView myaddr (myfrom.substr (0, lparen));
@@ -622,20 +622,16 @@ GNKSA :: do_check_from (const StringView   & from,
 	                bool                 strict)
 {
    int addrtype = 0;
-   debug ("From [" << from << "]");
 
    // split from 
    addr.clear ();
    name.clear ();
    int retval = split_from (from, addr, name, addrtype, strict);
-   debug ("Name [" << name << "]");
-   debug ("Addr [" << addr << "]");
 
    // check address 
    if (retval==OK && !addr.empty())
       retval = check_address (addr);
 
-   debug ("GNKSA [" << retval << ']');
    return retval;
 }
 
@@ -663,32 +659,6 @@ GNKSA :: get_short_author_name (const StringView& author)
 }
 
 
-/*****
-******
-*****/
-
-void
-GNKSA :: strip_realname (char * realname)
-{
-   // remove white spaces and wrapped parenthesis
-   StringView str (realname);
-   str.trim ();
-   if (!str.empty() && str.front()=='(') {
-      str = str.substr (str.begin()+1, NULL);
-      if (str.back() == ')')
-         str = str.substr (str.begin(), str.end()-1);
-   }
-
-   // strip out quotation marks
-   std::string buf;
-   for (StringView::const_iterator it=str.begin(), end=str.end(); it!=end; ++it)
-      if (*it != '"')
-         buf += *it;
-
-   // write it back
-   std::copy (buf.begin(), buf.end(), realname);
-}
-
 /*****
 ******
 *****/
@@ -840,9 +810,6 @@ GNKSA :: generate_message_id (const StringView& domain)
 {
    std::string s;
 
-   // start with '<'
-   s += '<';
-
    // add unique local part to message-id 
    s += "pan.";
    const time_t now (time(NULL));
@@ -857,9 +824,6 @@ GNKSA :: generate_message_id (const StringView& domain)
    // add domain
    s += domain.empty() ? default_domain : domain.to_string();
 
-   // end with '>'
-   s += '>';
-
    return s;
 }
 
@@ -910,41 +874,6 @@ GNKSA :: generate_references (const StringView   & references,
    return s;
 }
 
-/***
-****
-****  DATE
-****
-***/
-
-time_t
-GNKSA :: tzoffset_sec (time_t * now)
-{
-   struct tm gmt = *gmtime (now);
-   struct tm lt = *localtime (now);
-
-   int off = (lt.tm_hour - gmt.tm_hour) * 60 + lt.tm_min - gmt.tm_min;
-
-   if (lt.tm_year < gmt.tm_year)
-      off -= 24 * 60;
-   else if (lt.tm_year > gmt.tm_year)
-      off += 24 * 60;
-   else if (lt.tm_yday < gmt.tm_yday)
-      off -= 24 * 60;
-   else if (lt.tm_yday > gmt.tm_yday)
-      off += 24 * 60;
-
-   if (off >= 24 * 60)             /* should be impossible */
-      off = 23 * 60 + 59;     /* if not, insert silly value */
-   if (off <= -24 * 60)
-      off = -(23 * 60 + 59);
-   if (off > 13 * 60) 
-      off -= 24 * 60;
-   if (off < -12 * 60)
-      off += 24 * 60;
-
-   return off * 60;
-}
-
 /***
 ****  Signatures
 ***/
